// Precompiled headers for VC++
#include "stdafx.h"
#include "CreaturesConversation.h"

//-------------------------------------------------------------------
// Creatures::Session class
//-------------------------------------------------------------------
Creatures::Session::Session() :
	fDDEInstance(0),
	fVivariumString(0),
	fMacroString(0),
	fBlankString(0),
	fBrainActivityString(0),
	fSessionString(0),
	fConversation(0)
{
	initialiseDDE();
	fVivariumString = createDDEString("Vivarium");
	fMacroString = createDDEString("Macro");
	fBlankString = createDDEString(" ");
	fBrainActivityString = createDDEString("BrainActivity");
	fSessionString = createDDEString("AnyOldThing.EXE");
	fConversation = connectConversation(fVivariumString, fSessionString);		
}

Creatures::Session::~Session()
{
	cleanup();
}

char const* Creatures::Session::executeMacro(char const* macro)
{
	char const* retval = 0;
	const int length = strlen(macro) + 1;
	unsigned char *buffer = new unsigned char[length];
	memset(buffer, 0, length);
	memcpy(buffer, macro, length);

	DWORD result = 0;
	HDDEDATA macro_data = DdeCreateDataHandle(fDDEInstance, buffer, length, 0, fBlankString, CF_TEXT, 0);
	if(macro_data == 0)
	{
		handleError();
	}

	HDDEDATA data = DdeClientTransaction((LPBYTE)macro_data, 0xFFFFFFFF, fConversation, fBlankString, CF_TEXT, XTYP_POKE, 5000, &result);
	if(data == 0)
	{
		handleError();
	}

	HDDEDATA data2 = DdeClientTransaction(0, 0, fConversation, fMacroString, CF_TEXT, XTYP_REQUEST, 5000, &result);
	if(data2 == 0)
	{
		DdeFreeDataHandle(data);
		handleError();
	}

	// Reasonable size buffer - may need to dyanmically allocate for larger queries
	static unsigned char result_buffer[8 * 1024];
	memset(result_buffer, 0, sizeof(result_buffer));

	DdeGetData(data2, result_buffer, sizeof(result_buffer), 0);
	retval = (char const*)result_buffer;
	
	DdeFreeDataHandle(data);
	DdeFreeDataHandle(data2);
	delete[] buffer;
	return retval;
}

char const* Creatures::Session::getBrainActivity()
{
	char const* retval = 0;
	DWORD result = 0;
	HDDEDATA data = DdeClientTransaction(0, 0, fConversation, fBrainActivityString, CF_TEXT, XTYP_REQUEST, 5000, &result);
	if(data == 0)
	{
		handleError();
	}

	// Reasonable size buffer - may need to dyanmically allocate for larger queries
	static unsigned char result_buffer[8 * 1024];
	memset(result_buffer, 0, sizeof(result_buffer));

	DdeGetData(data, result_buffer, sizeof(result_buffer), 0);
	retval = (char const*)result_buffer;

	DdeFreeDataHandle(data);

	return retval;
}


void Creatures::Session::cleanup()
{
	if(fConversation != 0)
	{
		disconnectConversation(fConversation);		
		fConversation = 0;
	}
	if(fSessionString != 0)
	{
		freeDDEString(fSessionString);
		fSessionString = 0;
	}
	if(fBrainActivityString != 0)
	{
		freeDDEString(fBrainActivityString);
		fBrainActivityString = 0;
	}
	if(fBlankString != 0)
	{
		freeDDEString(fBlankString);
		fBlankString = 0;
	}
	if(fMacroString != 0)
	{
		freeDDEString(fMacroString);
		fMacroString = 0;
	}
	if(fVivariumString != 0)
	{
		freeDDEString(fVivariumString);
		fVivariumString = 0;
	}
	if(fDDEInstance != 0)
	{
		uninitialiseDDE();
		fDDEInstance = 0;
	}
}

void Creatures::Session::initialiseDDE()
{
	if(DdeInitialize(&fDDEInstance, &SessionCallback, APPCLASS_STANDARD, 0) != DMLERR_NO_ERROR)
	{
		handleError();
	}
}

void Creatures::Session::uninitialiseDDE()
{
	if(!DdeUninitialize(fDDEInstance))
	{
		handleError();
	}
}

HSZ Creatures::Session::createDDEString(char const* s)
{
	HSZ retval = DdeCreateStringHandle(fDDEInstance, s, CP_WINANSI);
	if(retval == 0)
	{
		handleError();
	}
	return retval;
}

void Creatures::Session::freeDDEString(HSZ s)
{
	if(!DdeFreeStringHandle(fDDEInstance, s))
	{
		handleError();
	}
}

HCONV Creatures::Session::connectConversation(HSZ service, HSZ topic)
{
	HCONV retval = DdeConnect(fDDEInstance, service, topic, 0);
	if(retval == 0)
	{
		handleError();
	}
	return retval;
}

void Creatures::Session::disconnectConversation(HCONV conversation)
{
	if(!DdeDisconnect(conversation))
	{
		handleError();
	}
}

void Creatures::Session::handleError()
{
	// Simple error handling...
	cleanup();
	throw DDEException();
}

HDDEDATA CALLBACK Creatures::SessionCallback(
    UINT uType,	// transaction type
    UINT uFmt,	// clipboard data format
    HCONV hconv,	// handle to the conversation
    HSZ hsz1,	// handle to a string
    HSZ hsz2,	// handle to a string
    HDDEDATA hdata,	// handle to a global memory object
    DWORD dwData1,	// transaction-specific data
    DWORD dwData2 	// transaction-specific data
   )
{
	HDDEDATA retval = 0;
	switch(uType)
	{
	case XTYP_ADVSTART:
	case XTYP_CONNECT:
		{
			retval = 0;
		}
		break;

	case XTYP_ADVREQ:
	case XTYP_REQUEST:
	case XTYP_WILDCONNECT:
		{
			retval = 0;
		}
		break;

	case XTYP_ADVDATA:
	case XTYP_EXECUTE:
	case XTYP_POKE:
		{
			retval = (HDDEDATA)DDE_FACK;
		}
		break;

	case XTYP_ADVSTOP:
	case XTYP_CONNECT_CONFIRM:
	case XTYP_DISCONNECT:
	case XTYP_ERROR:
	case XTYP_MONITOR:
	case XTYP_REGISTER:
	case XTYP_XACT_COMPLETE:
	case XTYP_UNREGISTER:
		{
			retval = 0;
		}
		break;

	default:
		{
			retval = 0;
		}
		break;
	}

	return retval;
}

